<?php

declare(strict_types=1);

namespace App\Interfaces\Repositories;

use Illuminate\Contracts\Pagination\LengthAwarePaginator;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\Eloquent\Model;

interface IBaseRepository
{
    /**
     * Return all data
     *
     * @param array $relations
     * @return Collection
     */
    public function all(array $relations = []): Collection;

    /**
     * Return all the data in descending order
     *
     * @param array $relations
     * @return Collection
     */
    public function latest(array $relations = []): Collection;

    /**
     * Create new record
     *
     * @param array $data
     * @return Model
     */
    public function create(array $data): Model;

    /**
     * Find record based on primary key
     *
     * @param int $id
     * @param array $relations
     * @return Model|null
     */
    public function findOrFail(int $id, array $relations = []): ?Model;

    /**
     * Find all records matching conditions
     *
     * @param array $conditions
     * @param array $relations
     * @return Collection
     */
    public function allWhere(array $conditions, array $relations = []): Collection;

    /**
     * Find first record matching conditions
     *
     * @param array $conditions
     * @param array $relations
     * @return Model|null
     */
    public function firstWhere(array $conditions, array $relations = []): ?Model;

    /**
     * Update record
     *
     * @param int $id
     * @param array $data
     * @return Model|null
     */
    public function update(int $id, array $data): ?Model;

    /**
     * Delete record
     *
     * @param int $id
     * @return bool
     */
    public function delete(int $id): bool;

    /**
     * Get All Records paginated
     *
     * @param int $paginate
     * @param array $conditions
     * @param array $relations
     * @return LengthAwarePaginator
     */
    public function paginate(int $paginate, array $conditions, array $relations = []): LengthAwarePaginator;

}
