<?php

declare(strict_types=1);

namespace App\Repositories;

use App\Interfaces\Repositories\IBaseRepository;
use Illuminate\Contracts\Pagination\LengthAwarePaginator;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\Eloquent\Model;

class BaseRepository implements IBaseRepository
{
    /**
     * @var Model $model
     */
    public Model $model;

    /**
     * Constructor
     *
     * @param Model $model
     */
    public function __construct(Model $model)
    {
        $this->model = $model;
    }

    /**
     * Return all data
     *
     * @return Collection
     */
    public function all(array $relations = []): Collection
    {
        return $this->model->query()->with($relations)->get();
    }

    /**
     * Return all the data in descending order
     *
     * @param array $relations
     * @return Collection
     */
    public function latest(array $relations = []): Collection
    {
        return $this->model->query()->with($relations)->latest()->get();
    }

    /**
     * Create new record
     *
     * @param array $data
     * @return Model
     */
    public function create(array $data): Model
    {
        return $this->model->query()->create($data);
    }

    /**
     * Find record based on primary key
     *
     * @param int $id
     * @param array $relations
     * @return Model
     */
    public function findOrFail(int $id, array $relations = []): Model
    {
        return $this->model->query()->with($relations)->findOrFail($id);
    }

    /**
     * Find all records matching conditions
     *
     * @param array $conditions
     * @param array $relations
     * @return Collection
     */
    public function allWhere(array $conditions, array $relations = []): Collection
    {
        return $this->model->query()->with($relations)->where($conditions)->get();
    }

    /**
     * Find first record matching conditions
     *
     * @param array $conditions
     * @param array $relations
     * @return Model|null
     */
    public function firstWhere(array $conditions, array $relations = []): ?Model
    {
        return $this->model->query()->where($relations)->where($conditions)->first();
    }

    /**
     * Update record
     *
     * @param int $id
     * @param array $data
     * @return Model|null
     */
    public function update(int $id, array $data): ?Model
    {
        $model = $this->model->query()->findOrFail($id);
        $model->update($data);
        return $model;
    }

    /**
     * Delete record
     *
     * @param int $id
     * @return bool
     */
    public function delete(int $id): bool
    {
        return $this->model->query()->findOrFail($id)->delete();
    }

    /**
     * Get All Records paginated
     *
     * @param int $paginate
     * @param array $conditions
     * @param array $relations
     * @return LengthAwarePaginator
     */
    public function paginate(int $paginate, array $conditions, array $relations = []): LengthAwarePaginator
    {
        return $this->model->query()->with($relations)->where($conditions)->paginate($paginate);
    }

    /**
     * Return all records order by based on conditions
     *
     * @param string $column
     * @param string $order
     * @return Collection|array
     */
    public function orderBy(string $column, string $order): Collection|array
    {
        return $this->model->query()->orderBy($column, $order)->get();
    }
}
